/****************************************************************************
**  MDHashTool - Message Digest (MD5, SHA-1, etc.) tool
**  Copyright (c) 2005  Charles Melhorn
**
**  This file is part of MDHashTool.
**
**  MDHashTool is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  MDHashTool is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**
**  Or see the GNU web site at: http://www.gnu.org
**
****************************************************************************/ 

// used to create file input stream - from nsUpdateService.js
const MODE_RDONLY   = 0x01;
const PERMS_FILE    = 0644;


const nsICryptoHash = Components.interfaces.nsICryptoHash;

var gPref = null;



// returns array resulting from match operation
// if valid, null if not

function isValidLinkFingerprint(href)
{
  var cs_array = null;
  var bValid = false;

  if ( cs_array = href.match(/#\!(md5|sha1)!([a-f0-9]+)/) )
   {
     // check hexvalue string length for corresponding hash type

     switch (cs_array[1])  
     {
       case "md5":   bValid = (cs_array[2].length == 32);
                     break;

       case "sha1":  bValid = (cs_array[2].length == 40);
                     break;

       default:      break;
     }
   }
  
  return (bValid ? cs_array : null);
 
}


//
// borrowed from nsUpdateService.js
//
function binaryToHex(input) {
  var result = "";
  for (var i = 0; i < input.length; ++i) {
    var hex = input.charCodeAt(i).toString(16);
    if (hex.length == 1)
      hex = "0" + hex;
    result += hex;
  }
  return result;
}


function mdhtAutoCSComponent()
{
  gPref = Components.classes["@mozilla.org/preferences-service;1"]
                    .getService(Components.interfaces.nsIPrefBranch);

}

mdhtAutoCSComponent.prototype = {
   
    QueryInterface: function (iid) {
       if ( !iid.equals(Components.interfaces.nsISupports) &&
            !iid.equals(Components.interfaces.nsIObserver) )
        {
            throw Components.results.NS_ERROR_NO_INTERFACE;
        }
       return this;
    },

    // nsIObserver
    observe: function (aSubject, aTopic, aData) {
        if (aTopic == "app-startup")
         {
           var observerService = Components.classes["@mozilla.org/observer-service;1"]
                                 .getService(Components.interfaces.nsIObserverService);
           observerService.addObserver(this, "dl-done",   false);
         }
        else if (aTopic == "dl-done")
         {
           
           if (gPref.getBoolPref("extensions.mdht.autochecksum"))
            {
              var dl = aSubject.QueryInterface(Components.interfaces.nsIDownload);
              var cs_array = null;

              if (cs_array = isValidLinkFingerprint(dl.source.spec) )
               {
                 var hash_type;
                 var reference_alg = cs_array[1].toLowerCase();
                 var reference_digest = cs_array[2].toLowerCase();

                 switch(reference_alg)
                  {
                    case "md5":  hash_type = nsICryptoHash.MD5;
                                 break;

                    case "sha1": hash_type = nsICryptoHash.SHA1;
                                 break;

                    default:     hash_type = nsICryptoHash.MD5;
                                 break;
                  }

                 var fileStream = Components.classes["@mozilla.org/network/file-input-stream;1"].
                                  createInstance(Components.interfaces.nsIFileInputStream);
                 fileStream.init(dl.targetFile, MODE_RDONLY, PERMS_FILE, 0);

                 var hashComp = Components.classes["@mozilla.org/security/hash;1"].
                                createInstance(nsICryptoHash);

                 hashComp.init(hash_type);

                 hashComp.updateFromStream(fileStream, -1);
                 var checksum = binaryToHex(hashComp.finish(false));

                 fileStream.close();

                 if (reference_digest != checksum)
                  {
                    var lfInfo = {};

                    lfInfo.urlspec = dl.source.spec;
                    lfInfo.urlspec = lfInfo.urlspec.substring(0, lfInfo.urlspec.indexOf("#!"));

                    lfInfo.alg = cs_array[1];
                    lfInfo.refdigest = cs_array[2];
                    lfInfo.compdigest = checksum;

                    // to preserve the context of the passed object

                    lfInfo.wrappedJSObject = lfInfo;

//        var ps = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
//                          .getService(Components.interfaces.nsIPromptService);
//        ps.alert(null, 
//                 "checksum verification failed", 
//                 lfInfo.compdigest);



//                    window.openDialog('chrome://mdhashtool/content/dialogAutoCSAlert.xul',
//                                      'mdhtACVAlert','chrome,centerscreen,modal', lfInfo);

                    var ww = Components.classes["@mozilla.org/embedcomp/window-watcher;1"]
                             .getService(Components.interfaces.nsIWindowWatcher);

                    ww.openWindow(ww.activeWindow, "chrome://mdhashtool/content/dialogAutoCSAlert.xul", 
                                  "mdhtACVAlert", 'chrome,centerscreen,modal', lfInfo);

                  }

               } // end if valid link fingerprint
            }
         }
        else
          throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    }
}


// mdhtAutoCSComponent Class Factory
var mdhtAutoCSFactory = {

    createInstance: function (outer, iid) {
      if (outer != null)
        throw Components.results.NS_ERROR_NO_AGGREGATION;

      return (new mdhtAutoCSComponent()).QueryInterface(iid);
    }
}
    

// mdhtAutoCSComponent Module
var Module = {
    
    myCID: Components.ID("{e226e074-7db7-4f67-8380-a975b9c55481}"),
    myProgID: "@mozilla.org/mdht_autocs;1",

    registerSelf: function (compMgr, fileSpec, location, type) {

        var bDeferRegistration = false;  // no need to defer

        if (bDeferRegistration) {
            // if prerequisite components are not available,
            // defer registration and wait for them to be registered

            throw Components.results.NS_ERROR_FACTORY_REGISTER_AGAIN;
        }

        compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
        compMgr.registerFactoryLocation(this.myCID,
                                        "MDHT AutoCS JS Component",
                                        this.myProgID,
                                        fileSpec,
                                        location,
                                        type);
          
        // Make mdhtAutoCSComponent a startup observer
        var categoryManager = Components.classes["@mozilla.org/categorymanager;1"]
                              .getService(Components.interfaces.nsICategoryManager);

        categoryManager.addCategoryEntry("app-startup",
                                         "mdhtAutoCS component",
                                         "service," + this.myProgID, 
                                          true, true);
    },

    unregisterSelf: function (compMgr, fileSpec, location) {

        compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
        compMgr.unregisterFactoryLocation(this.myCID,
                                          fileSpec);

        var categoryManager = Components.classes["@mozilla.org/categorymanager;1"]
                              .getService(Components.interfaces.nsICategoryManager);

        categoryManager.deleteCategoryEntry("app-startup",
                                         "mdhtAutoCS component", 
                                          true);
    },

    getClassObject : function (compMgr, cid, iid) {

        if (!cid.equals(this.myCID))
          throw Components.results.NS_ERROR_NO_INTERFACE

        if (!iid.equals(Components.interfaces.nsIFactory))
          throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

        return mdhtAutoCSFactory;
    },

    canUnload: function(compMgr) {
        return true;
    }
}; // END Module

/* entrypoint */
function NSGetModule(compMgr, fileSpec) { return Module; }